/////////////////////////////////////////////////////////////////////////////
///
/// @file
///
/// Configuration for the time-t map for an infectious disease model.
///
/// @author Pawel Pilarczyk
///
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 1997-2014 by Pawel Pilarczyk.
//
// This file is part of my research software package. This is free software:
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation,
// either version 3 of the License, or (at your option) any later version.
//
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this software; see the file "license.txt". If not,
// please, see <https://www.gnu.org/licenses/>.

// Started on February 9, 2008. Last revision: February 9, 2008.


#ifndef _CMGRAPHS_CONFIG_H_
#define _CMGRAPHS_CONFIG_H_


// Use single parameter points instead of parameter intervals.
// Comment this setting out for the final computations.
//#define CONFIG_PARAMPOINTS

// Do not send individual parameter boxes for precomputing
// the Conley indices.
//#define CONFIG_NOPREPROCESSING

// Do preprocessing only and skip the matching part.
#define CONFIG_ONLYPREPROCESSING

// Use the procedure for computing the Morse decomposition
// instead of computing the invariant part of the set,
// and also don't determine connections between Morse sets.
// In this way, the connections between Morse sets are lost,
// but the invariant part to subdivide may be substantially smaller.
// Comment this setting out for the final computations
// to get full information.
//#define CONFIG_NOCONNECTIONS

// Use the default bounding box for ODE integration in order to prevent
// iterating trajectories far beyond the phase space of interest,
// and, in particular, to avoid running out of the scope of integer
// coordinates of boxes when determining the images of boxes.
// WARNING: This technique may lead to the construction of combinatorial
// cubical multivalued maps that are not acyclic, that is, do not satisfy
// the necessary assupmtions for the computation of the homomorphism
// induced in homology (for the Conley index map).
// NOTE: If using the approximate integration of the ODE, this technique
// works in a different way, by fading the vector field outside the box,
// so it is safe to use it in that setting.
//#define CONFIG_ODEBOUNDINGBOX

// Make a correction to the isolating neiborhood after subdivision.
// This is necessary for integrating ODEs using the time-t map
// with variable t, because an isolating neighborhood for one t
// need not be an isolating neighborhood for another t, so it is only taken
// as a hint, and those cubes from the exit set whose images intersect
// the isolating neighborhood are added to the isolating neighborhood,
// like in an algorithm for the construction of an index pair.
#define CONFIG_ODEVART

// Define the X coordinate for plotting the Morse sets in the PNG files.
// Note: Plotting the amount of infected individuals.
#define PLOT_X_COORD 0

// Define the Y coordinate for plotting the Morse sets in the PNG files.
// Note: Plotting the amount of infected individuals.
#define PLOT_Y_COORD 2


// --------------------------------------------------
// -------------- THE PARAMETER RANGES --------------
// --------- AND THE PHASE SPACE SELECTION ----------
// --------------------------------------------------

#include "p_inf.h"
using namespace custom::InfModel;


// --------------------------------------------------
// --------------- THE MAP SELECTION ----------------
// --------------------------------------------------

#include "m_inf.h"

/// Defines the type of the map to be used for the computations.
/// The corresponding header file must be included prior to this definition.
typedef MapInf theMapType;
#define _MAP_COMP_TIGHT_


// --------------------------------------------------
// -------------- KNOWN CONLEY INDICES --------------
// --------------------------------------------------

#include "indknown.h"
using namespace custom::noKnownIndices;


// --------------------------------------------------
// ---- POST-PROCESSING OF MORSE DECOMPOSITIONS -----
// --------------------------------------------------

#include "procmdec.h"
//#include "procsave.h"
using namespace custom::noProcMorseDec;
//using namespace custom::saveMorseDec;


// --------------------------------------------------
// ------------- COMPATIBILITY CONTROL --------------
// --------------------------------------------------

/// The control number that is used to confirm the compatibility
/// between the coordinator and workers.
/// Please, increase this number by 1 every time the configuration
/// of the program changes substantially (e.g., different parameter ranges,
/// different numbers of subintervals, or different maps are chosen).
const int controlNumber = 31130;


#endif // _CMGRAPHS_CONFIG_H_

