/////////////////////////////////////////////////////////////////////////////
///
/// @file m_periodic.h
///
/// A sample silly periodic map.
///
/// This is a sample periodic map on a multi-dimensional torus
/// which may be used for testing the capability of the software
/// to deal with periodic boundary conditions.
///
/// @author Pawel Pilarczyk
///
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 1997-2014 by Pawel Pilarczyk.
//
// This file is part of my research software package. This is free software:
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation,
// either version 3 of the License, or (at your option) any later version.
//
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this software; see the file "license.txt". If not,
// please, see <https://www.gnu.org/licenses/>.

// Started on October 3, 2008. Last revision: October 3, 2008.


#ifndef _CMGRAPHS_PERIODIC_H_
#define _CMGRAPHS_PERIODIC_H_


// include local header files
#include "maptype.h"
#include "typeintv.h"


// --------------------------------------------------
// ---------- a sample silly periodic map -----------
// --------------------------------------------------

/// This class defines a sample periodic map on a multi-dimensional torus
/// which may be used for testing the capability of the software
/// to deal with periodic boundary conditions.
/// The map is defined on the interval [0,1].
class MapPeriodic: public MapType
{
public:
	/// The default constructor.
	MapPeriodic ();

	/// Computes the image of a box whose left and right coordinates
	/// are given. Fills in the images with the left and right
	/// coordinates of the image box.
	void compute (const double *xleft, const double *xright,
		double *yleft, double *yright, int dim,
		const spcCoord *coord, int subdiv) const;

}; /* class MapPeriodic */

// --------------------------------------------------

inline MapPeriodic::MapPeriodic ()
{
	testIntervals ();
	return;
} /* MapPeriodic::MapPeriodic */

inline void MapPeriodic::compute (const double *xleft, const double *xright,
	double *yleft, double *yright, int dim, const spcCoord *, int) const
{
	for (int i = 0; i < dim; ++ i)
	{
		if (xleft [i] < 0)
		{
			ERRORMSG << "Wrong left value: " << xleft [i] <<
				", should be 0.\n" << THROW
		}
		if (xright [i] > 1)
		{
			ERRORMSG << "Wrong right value: " << xright [i] <<
				", should be 1.\n" << THROW
		}
		const double shift = 0.3;
		interval x = interval (xleft [i], xright [i]) + shift;
		if (x. leftBound () >= 1)
			x -= 1;
		interval fx = 0.5 + 5 * x * (2 * x - 1) * (x - 1) - shift;
		yleft [i] = fx. leftBound ();
		yright [i] = fx. rightBound ();
	}

	// swich the rounding direction to the neutral one
	resetRounding ();
	return;
} /* MapPeriodic::compute */


#endif // _CMGRAPHS_PERIODIC_H_

