#!/usr/bin/perl

#############################################################################
#
# This Perl script creates a list of single cubes which are not in any
# phenotype class of two or more cubes. The size of the box and the sets
# of cubes in nontrivial classes are determined on the basis of files
# saved by the "cmgraphs" software.
# Call this script with: filename.dat cubprefix outname.cub [outprefix].
# Default file names are assumed if these arguments are omitted.
#
#############################################################################

# Copyright (C) 1997-2014 by Pawel Pilarczyk.
#
# This file is part of my research software package. This is free software:
# you can redistribute it and/or modify it under the terms of the GNU
# General Public License as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; see the file "license.txt". If not,
# please, see <https://www.gnu.org/licenses/>.

# Started on April 24, 2008. Last revision: May 22, 2008.


# assume a fixed suffix for names of files with cubes
my $cubsuffix = ".cub";

# set up the file names
my $datname = "coord.dat";
$datname = $ARGV[0] if (scalar (@ARGV) > 0);
my $cubprefix = "cont";
$cubprefix = $ARGV[1] if (scalar (@ARGV) > 1);
my $outname = "single${cubsuffix}";
$outname = $ARGV[2] if (scalar (@ARGV) > 2);
my $outprefix = "";
$outprefix = $ARGV[3] if (scalar (@ARGV) > 3);

# determine the dimension and size of the large box
my $maxcounters = '';
my $datfile;
open ($datfile, $datname) or die "Unable to open '$datname'";
while (<$datfile>)
{
	next unless m/subdivisions: ([\d x]+)/;
	$maxcounters = $1;
}
close $datfile;
die "Unable to find ranges of cubes in '$datname'.\n" unless $maxcounters;

# split the string representation of counters into an array
my @maxcounters = split (/\s*x\s*/, $maxcounters);
my $dim = scalar (@maxcounters);
my @counters = map {0} @maxcounters;

# show the message what is to be done
print "Full range: ", join (" x ", @maxcounters), ", dim = $dim.\n";

# read sets of cubes from the disk
print "Reading files: ";
my $maxnum = 0;
my %cubes;
foreach my $filename (glob "${cubprefix}*${cubsuffix}")
{
	next unless $filename =~ m/${cubprefix}(\d+)${cubsuffix}/;
	my $num = $1;
	$maxnum = ($num + 1) if ($maxnum <= $num);
	my $file;
	open ($file, $filename) or next;
	print "$filename ";
	my $counter = 0;
	while (<$file>)
	{
		next unless m/\(([\d, ]+)\)/;
		my $cube = $1;
		$cube =~ s/[ ,]/,/g;
		$cubes{$cube} = 1;
		++ $counter;
	}
	close $file;
	print "($counter) ";
}
print "Total ", scalar (keys (%cubes)), " cubes.\n";

# determine single cubes as the ones missing in the set of read ones
my @single;
while (1)
{
	# add the cube to the list of single ones if necessary
	my $cube = join (",", @counters);
	push (@single, $cube) unless ($cubes{$cube} == 1);

	# take the next counter value
	my $pos = 0;
	while ($pos < $dim)
	{
		++ $counters [$pos];
		last if ($counters [$pos] < $maxcounters [$pos]);
		$counters [$pos] = 0;
		++ $pos;
	}
	last if ($pos == $dim);
}

# save single cubes to a file
my $countsingle = scalar (@single);
print "Saving ${countsingle} single cubes to ${outname}.\n";
my $outfile;
open ($outfile, ">$outname") or die "Can't create file '$outname'";
print $outfile "; $countsingle single cubes.\n";
foreach my $coords (@single)
{
	print $outfile "(${coords})\n";
}
close $outfile;

# save each single cube to a single file
if ($outprefix)
{
	for (my $i = 0; $i < scalar (@single); ++ $i)
	{
		# prepare the text number with the padding zeros
		my $num = $maxnum + $i;
		my $number = "$num";
		my $ten = 10;
		while (($num < $ten) && ($ten < $maxnum + scalar ($single)))
		{
			$number = "0$number";
			$ten *= 10;
		}
		my $name = "${outprefix}${number}${cubsuffix}";

		# create a file for the single cube
		my $outfile;
		open ($outfile, ">$name")
			or die "Can't create file '$name'";

		# write the coordinates to the file
		$coords = $single [$i];
		print $outfile "(${coords})\n";

		# close the file
		close $outfile;
	}
}
